// Description: a node that stores and manipulates a character string
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//</ul>
//
// Category: Math
// Author: Alex Hill
// Revision: 04/02/03
//
#include <ygNetMessages.h>
#include "String.h"

extern "C" ygNode* construct_String(const char* name,bool master) { return new String(name,master); }


String::String(const char* name,bool master) : ygNode(name,master)
    {
    setClassName("string");
    netString.setCaseSensitive();
    lastString.setCaseSensitive();
    length = 0;
    addNetMessage("data",&netString,YG_NET_STRING);
    addNetMessage("length",&length,YG_NET_INT);
    addEvent("changed");
    }

String::~String(void)
    {
    }


void String::reset(void)
    {
    resetNetMessage("data");
    lastString.clear();
    ygNode::reset();
    }


void String::message(const ygMessage& msg)
    {
    //set string string to be generated
    if (msg == "data")
        {
        checkChanged();
        }
    //concatenate onto the string
    else if (msg == "concatenate")
        {
        if (msg.numArgs() > 0)
            {
            netString += msg.stringArg(0);
            checkChanged();
            }
        else
            msg.error(name(),"(wrong number of arguments)");
        }
    //erase from optional starting point and characters
    else if (msg == "erase")
        {
        if (msg.numArgs() > 0)
            {
            int pos = msg.intArg(0);
            if (msg.numArgs() > 1)
                netString.erase(pos,msg.intArg(1));
            else
                netString.erase(pos);
            }
        else
            netString.erase(0);
        checkChanged();
        }
    //length is provided for event access only
    else if (msg == "length")
        {
        return;
        }
    else
        ygNode::message(msg);
    }

void String::checkChanged(void)
    {
    if (netString != lastString)
        {
        length = netString.length();
        //the string has changed
        eventOccurred("changed");
        lastString = netString;
        }
    }
