// Revision: 11/01/01 Dave Pape

#include <Performer/pf.h>
#include <Performer/pfdu.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pr/pfGeoState.h>
#include <Performer/pr/pfMaterial.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pf/pfGroup.h>
#include <Performer/pf/pfTraverser.h>
#include <math.h>


int transparencyType = PFTR_ON;

void traverseGraph(pfNode *node)
{
 int i;
 if (node->isOfType(pfGeode::getClassType()))
    {
    pfGeode *geode = (pfGeode *)node;
    for (i=0; i < geode->getNumGSets(); i++)
        {
        pfGeoSet *gset = geode->getGSet(i);
        gset->getGState()->setMode(PFSTATE_TRANSPARENCY, transparencyType);
        if (transparencyType != PFTR_OFF)
            gset->setDrawBin(PFSORT_TRANSP_BIN);
        else
            gset->setDrawBin(PFSORT_OPAQUE_BIN);
        }
    }
 else if (node->isOfType(pfGroup::getClassType()))
    {
    pfGroup *group = (pfGroup *)node;
    for (i=0; i < group->getNumChildren(); i++)
        traverseGraph(group->getChild(i));
    }
}


main(int argc, char **argv)
{
 pfNode *node;
 int opt=1,loop=1;

 if (argc < 3)
    {
    fprintf(stderr, "usage: %s [-b (on|blend|off)]\n"
        "\tinfile outfile\n", argv[0]);
    exit(1);
    }

 pfInit();

 while (loop)
    {
    if (!strcmp(argv[opt],"-b"))
        {
        opt++;
        if (!strcmp(argv[opt],"on"))
            transparencyType = PFTR_ON;
        else if (!strcmp(argv[opt],"blend"))
            transparencyType = PFTR_BLEND_ALPHA;
        else if (!strcmp(argv[opt],"off"))
            transparencyType = PFTR_OFF;
        opt++;
        }
    else
        loop=0;
    }

 if (!(pfdInitConverter(argv[opt])))
    exit(1);
 if (!(pfdInitConverter(argv[opt+1])))
    exit(1);

 pfMultiprocess(0);
 pfConfig();

 node = pfdLoadFile(argv[opt]);

 traverseGraph(node);

 pfdStoreFile(node, argv[opt+1]);
}
